package edu.odu.cs.zeil.AlgAE.Client.SourceViewer;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

public class SourcePane extends JPanel implements SourceViewer
{

  //private static Logger logger = Logger.getLogger(SourcePane.class.getName());

  
  
  private JTextArea mainPanel;
  private ArrayList<SourceFile> files;
  private JComboBox fileSelector;
  private int selected;
  private SourceLocation displaying;
  private JComboBox fontSizeSelector;
  

    
   
  public SourcePane()
  {
	  setLayout(new BorderLayout());

	  mainPanel = new JTextArea();
	  mainPanel.setEditable(false);
	  mainPanel.setSelectedTextColor(Color.red);
	  
	  JScrollPane scrolled = new JScrollPane(mainPanel);
    
	  add (scrolled, BorderLayout.CENTER);

	  JPanel controls = new JPanel();
	  add (controls, BorderLayout.SOUTH);
	  
	  fileSelector = new JComboBox();
	  controls.add (fileSelector);

	  files = new ArrayList<SourceFile>();

	  Integer[] fontSizes = {8, 9, 10, 11, 12, 14, 16, 18, 20, 22, 24, 28, 32, 36};
	  fontSizeSelector = new JComboBox(fontSizes);
	  fontSizeSelector.setSelectedIndex(4);
	  setFontSize(12);
	  fontSizeSelector.addActionListener(
			  new ActionListener() {
				
				@Override
				public void actionPerformed(ActionEvent e) {
					Integer fontSize = (Integer)fontSizeSelector.getSelectedItem();
			        setFontSize(fontSize);
				}
			}
			  );
	  
	  controls.add(new JLabel("  Size:"));
	  controls.add(fontSizeSelector);
	  
	  selected = -1;
	  displaying = null;
	  setFocusable(false);
  }

  
  public void display (SourceLocation sloc)
  {
	  if (displaying != null && sloc.equals(displaying)) {
		  return;
	  }
	  if (sloc.fileName.length() == 0)
		  return;
	  
	  int found = -1;
	  for (int i = 0; i < files.size() && found < 0; ++i) {
		  if (sloc.getResourceName().equals(files.get(i).getResourceName())) {
			  found = i;
		  }
	  }
	  if (found < 0) {
		  SourceFile newEntry = new SourceFile (sloc.getContainer(), sloc.getFileName());
		  found = files.size();
		  files.add(newEntry);
		  fileSelector.addItem (sloc.getResourceName());
		  found = files.size()-1;
	  }
	  SourceFile selectedEntry = files.get(found);
	  if (found != selected) {
		  selected = found;
		  fileSelector.setSelectedIndex(selected);
		  mainPanel.setText(selectedEntry.getContents());
	  }
	  if (sloc.lineNumber > 0 && sloc.lineNumber <= selectedEntry.getLineOffsets().size()-1) {
		  displaying = sloc;
		  int start = selectedEntry.getLineOffsets().get(sloc.lineNumber-1);
		  int lineNum = sloc.lineNumber;
		  int stop = start; 
		  while (lineNum < selectedEntry.getLineOffsets().size() && stop == start) {
			  stop = selectedEntry.getLineOffsets().get(lineNum);
			  ++lineNum;
		  }
		  mainPanel.setCaretPosition(start);
		  mainPanel.moveCaretPosition(stop-1);
		  mainPanel.requestFocus();
		  mainPanel.repaint();
	  }
  }


  
  public void setFontSize (int size)
  {
    Font textFont = new Font("Courier", Font.PLAIN, size);
    mainPanel.setFont (textFont);
  }

  
  public Dimension getPreferredSize() {
    return new Dimension (400,50);
  }


  
}
