package edu.odu.cs.zeil.AlgAE.Snapshot;

import java.awt.Color;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;
import edu.odu.cs.zeil.AlgAE.Animation;
import edu.odu.cs.zeil.AlgAE.Client.SourceViewer.SourceLocation;
import edu.odu.cs.zeil.AlgAE.Snapshot.Rendering.CanBeRendered;
import edu.odu.cs.zeil.AlgAE.Snapshot.Rendering.Renderer;

/**
 * @author zeil
 *
 */
public class TestSnapshot extends TestCase {

    public TestSnapshot(String testName) {
        super(testName);
    }

    public static void main( String[] args ) throws Exception {
        TestRunner.run( suite() );
    }

    public static Test suite() {
        return new TestSuite(TestSnapshot.class);
    }
    
    private static Animation a;
    public void setUp() {
    	a = new Animation("dummy", false) {
			@Override
			public void buildMenu() {
			}

			@Override
			public String about() {
				return null;
			}
    	};
    	a.registerInstance(Thread.currentThread());
    }
    
    
	private static void checkvar (HashMap<Identifier, LinkedList<Entity> > entities, Object obj, int len, String label, String value) {
		LinkedList<Entity> list = entities.get(Identifier.getIdentifierFor(obj, a));
		assertNotNull(list);
		assertEquals (len, list.size());
		Entity e = list.getFirst();
		if (label != null)
			assertEquals (label, e.getLabel());
		if (value != null)
			assertEquals(value, e.getValue());
		
	}
	

    
	public static void test_emptySnap() {
		ActivationStack callStack = new ActivationStack(a);
		Snapshot snap = new Snapshot("description", new SourceLocation(TestSnapshot.class, "foo.java", 23), callStack);
		HashMap<Identifier, LinkedList<Entity> > entities =  snap.getEntities();
		assertEquals (2, entities.keySet().size());
		checkvar (entities, callStack, 1, null, null);
	}

	public static void test_Snap_global() {
		ActivationStack callStack = new ActivationStack(a);
		String fortyTwo = "42";
		callStack.globalVar("foo", fortyTwo);
		Snapshot snap = new Snapshot("description", new SourceLocation(TestSnapshot.class, "foo.java", 23), callStack);
		HashMap<Identifier, LinkedList<Entity> > entities =  snap.getEntities();
		assertEquals (3, entities.keySet().size());
		checkvar (entities, callStack, 1, null, null);
		checkvar (entities, fortyTwo, 1, "foo", "42");
	}

	private static class String2 implements Renderer<String2>, CanBeRendered<String2> {
		
		public String s;
		
		public String2 (String ss) {
			s = ss;
		}

		@Override
		public Renderer<String2> getRenderer() {
			return this;
		}

		@Override
		public String getValue(String2 obj) {
			return s;
		}

		@Override
		public Color getColor(String2 obj) {
			return Color.blue;
		}

		@Override
		public List<Component> getComponents(String2 obj) {
			LinkedList<Component> c = new LinkedList<Component>();
			c.add(new Component(s, "comp"));
			return c;
		}

		@Override
		public List<Connection> getConnections(String2 obj) {
			return new LinkedList<Connection>();
		}

		@Override
		public int getMaxComponentsPerRow(String2 obj) {
			return 1;
		}
		
	}
	
	public static void test_Snap_components() {
		ActivationStack callStack = new ActivationStack(a);
		String fortyTwo = "42";
		callStack.globalVar("foo", fortyTwo);
		String twelve = "12";
		String2 dozen = new String2(twelve);
		callStack.globalVar("dozen", dozen);
		Snapshot snap = new Snapshot("description", new SourceLocation(TestSnapshot.class, "foo.java", 23), callStack);
		HashMap<Identifier, LinkedList<Entity> > entities =  snap.getEntities();
		assertEquals (5, entities.keySet().size());
		checkvar (entities, callStack, 1, null, null);
		checkvar (entities, fortyTwo, 1, "foo", "42");
		checkvar (entities, dozen, 1, "dozen", "12");
		checkvar (entities, twelve, 1, "comp", "12");
	}

	public static void test_Snap_connections() {
		ActivationStack callStack = new ActivationStack(a);
		String fortyTwo = "42";
		callStack.globalVar("foo", fortyTwo);
		String twelve = "12";
		String2 dozen = new String2(twelve);
		callStack.globalRefVar("dozen", dozen);
		Snapshot snap = new Snapshot("description", new SourceLocation(TestSnapshot.class, "foo.java", 23), callStack);
		HashMap<Identifier, LinkedList<Entity> > entities =  snap.getEntities();
		assertEquals (6, entities.keySet().size());
		checkvar (entities, callStack, 1, null, null);
		checkvar (entities, fortyTwo, 1, "foo", "42");
		checkvar (entities, dozen, 1, null, "12");
		checkvar (entities, twelve, 1, "comp", "12");
	}

	public static void test_Snap_alias() {
		ActivationStack callStack = new ActivationStack(a);
		String fortyTwo = "42";
		callStack.globalVar("foo", fortyTwo);
		String2 dozen = new String2(fortyTwo);
		callStack.globalVar("dozen", dozen);
		Snapshot snap = new Snapshot("description", new SourceLocation(TestSnapshot.class, "foo.java", 23), callStack);
		HashMap<Identifier, LinkedList<Entity> > entities =  snap.getEntities();
		assertEquals (4, entities.keySet().size());
		checkvar (entities, callStack, 1, null, null);
		checkvar (entities, fortyTwo, 2, "foo", "42");
		checkvar (entities, dozen, 1, "dozen", "42");
	}
}
