#ifndef VISIBLE_POINTER_H
#define VISIBLE_POINTER_H

#include <algae/config.h>
#include <iostream>
#include <iomanip>
#include <string>

#include <algae/algae.h>
#include <algae/visible.h>


template  <class T, AlgAE::Directions dir>
class VisibleConstPointer: public Visible
{
protected:
  const T* theData;
  AlgAE::Color _color;
public:
  VisibleConstPointer();
  VisibleConstPointer(AlgAE::Color c);
  VisibleConstPointer(const T* p, AlgAE::Color c = AlgAE::Magenta);
  VisibleConstPointer(const T* p, const std::string& name,
		      AlgAE::Color c = AlgAE::Magenta);
  VisibleConstPointer (const VisibleConstPointer<T, dir>& x);
  
  const VisibleConstPointer<T, dir>&
    operator= (const VisibleConstPointer<T, dir>& x);
  const VisibleConstPointer<T, dir>& operator= (const T* x);

  virtual ~VisibleConstPointer();

  const T& operator*() const {return *theData;}
  const T* operator->() const {return theData;}
  
  operator const T*() const  {return theData;}
  
  bool operator< (const VisibleConstPointer<T, dir>& ptr) const
    {
      return theData < ptr.theData;
    }
  
  bool operator== (const VisibleConstPointer<T, dir>& ptr) const
    {
      return theData == ptr.theData;
    }
  
  bool operator!= (const VisibleConstPointer<T, dir>& ptr) const
    {
      return theData != ptr.theData;
    }
  
  bool operator== (const T* ptr) const
    {
      return theData == ptr;
    }
  
  bool operator!= (const T* ptr) const
    {
      return theData != ptr;
    }

  // Required for animation
  virtual void touchAllPointers() const;
  virtual void writeText(std::ostream& out) const;
};



template  <class T, AlgAE::Directions dir>
class VisiblePointer: public VisibleConstPointer<T, dir>
{
  typedef VisibleConstPointer<T, dir> Super;
public:
  VisiblePointer(): VisibleConstPointer<T, dir>(AlgAE::Magenta) {}

  VisiblePointer(AlgAE::Color c): VisibleConstPointer<T, dir>(c) {}

  VisiblePointer(T* p, AlgAE::Color c = AlgAE::Magenta)
    : VisibleConstPointer<T, dir>(p,c) {}
  VisiblePointer(T* p, const std::string& name,
		 AlgAE::Color c = AlgAE::Magenta)
    : VisibleConstPointer<T, dir>(p,name,c) {}

  VisiblePointer (const VisiblePointer<T, dir>& x)
    : VisibleConstPointer<T, dir>(x) {}
  
  const VisiblePointer<T, dir>&
    operator= (const VisibleConstPointer<T, dir>& x);

  const VisiblePointer<T, dir>& operator= (const T* x);

  ~VisiblePointer();
  
  T& operator*() const;
  T* operator->() const;
  
  operator T*() const   {return (T*)Super::theData;}
  
  void del ()
    {
      T* d = (T*)(Super::theData);
      delete d;
      Super::theData = 0;
    }
  
};

template  <class T, AlgAE::Directions dir>
T& VisiblePointer<T, dir>::operator*() const {return (T&)(*Super::theData);}

template  <class T, AlgAE::Directions dir>
T* VisiblePointer<T, dir>::operator->() const {return (T*)Super::theData;}


template  <class T, AlgAE::Directions dir>
VisibleConstPointer<T, dir>::VisibleConstPointer()
  : Visible(AlgAE::Magenta), _color(AlgAE::Magenta), theData(0) {}

template  <class T, AlgAE::Directions dir>
VisibleConstPointer<T, dir>::VisibleConstPointer(AlgAE::Color c)
  : Visible(c), _color(c), theData(0) {}

template  <class T, AlgAE::Directions dir>
VisibleConstPointer<T, dir>::VisibleConstPointer(const T* p, AlgAE::Color c)
  : Visible(c), _color(c), theData(p) {}

template  <class T, AlgAE::Directions dir>
VisibleConstPointer<T, dir>::VisibleConstPointer
 (const T* p, const std::string& name, AlgAE::Color c):
   Visible(c), _color(c), theData(p)
{setName (name);}

template  <class T, AlgAE::Directions dir>
VisibleConstPointer<T, dir>::VisibleConstPointer
 (const VisibleConstPointer& x):
   Visible(x._color), _color(x._color), theData(x.theData) {}

template  <class T, AlgAE::Directions dir>
const VisibleConstPointer<T, dir>& VisibleConstPointer<T, dir>::
  operator= (const VisibleConstPointer& x)
{
  theData = x.theData;
  return *this;
}

template  <class T, AlgAE::Directions dir>
const VisiblePointer<T, dir>& VisiblePointer<T, dir>::
  operator= (const VisibleConstPointer<T, dir>& x)
{
  VisibleConstPointer<T,dir>::operator= (x);
  return *this;
}

template  <class T, AlgAE::Directions dir>
const VisibleConstPointer<T, dir>& VisibleConstPointer<T, dir>::
  operator= (const T* x)
{
  theData = x;
  return *this;
}

template  <class T, AlgAE::Directions dir>
const VisiblePointer<T, dir>& VisiblePointer<T, dir>::operator= (const T* x)
{
  Super::theData = x;
  return *this;
}

template  <class T, AlgAE::Directions dir>
VisibleConstPointer<T, dir>::~VisibleConstPointer() {}

template  <class T, AlgAE::Directions dir>
VisiblePointer<T, dir>::~VisiblePointer() {}

template  <class T, AlgAE::Directions dir>
void VisibleConstPointer<T, dir>::touchAllPointers() const
{
  touch(theData, dir);
}

template  <class T, AlgAE::Directions dir>
void VisibleConstPointer<T, dir>::writeText(std::ostream& out) const
{
  out << ' ';
}




#endif
