#include <algae/algae.h>
#include <algae/astream.h>
#include <algae/messages.h>
#include <algae/sockstr.h>
#include <algae/visible.h>

#ifdef USE_WINSOCK2
#include <winsock2.h>
#include <io.h>
#else
#include <arpa/inet.h>
#include <netinet/in.h>
#include <unistd.h>
#include <netdb.h>
#include <sys/socket.h>
#endif

#include <sstream>

#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <map>
#include <list>
#include <algorithm>

#ifndef _MSC_VER
// Only timer functions I can find in Visual C++ are associated with
// windows.  Can I set a timer in a console application?
#include <signal.h>
#include <sys/time.h>
#include <errno.h>
#endif

using namespace std;

#define DEBUG if (dbgstream != 0) (*dbgstream)

  const AlgAE::Color AlgAE::Black       = {0, 0, 0};
  const AlgAE::Color AlgAE::Red         = {255, 0, 0};
  const AlgAE::Color AlgAE::PaleRed     = {255, 150, 150};
  const AlgAE::Color AlgAE::Green       = {0, 255, 0};
  const AlgAE::Color AlgAE::PaleGreen   = {150, 255, 150};
  const AlgAE::Color AlgAE::Blue        = {0, 0, 255};
  const AlgAE::Color AlgAE::PaleBlue    = {150, 150, 255};
  const AlgAE::Color AlgAE::Yellow      = {255, 255, 0};
  const AlgAE::Color AlgAE::PaleYellow  = {255, 255, 150};
  const AlgAE::Color AlgAE::Magenta     = {255, 0, 255};
  const AlgAE::Color AlgAE::PaleMagenta = {255, 150, 255};
  const AlgAE::Color AlgAE::Cyan        = {0, 255, 255};
  const AlgAE::Color AlgAE::PaleCyan    = {150, 255, 255};
  const AlgAE::Color AlgAE::DarkGray    = {128, 128, 128};
  const AlgAE::Color AlgAE::MedGray     = {192, 192, 192};
  const AlgAE::Color AlgAE::White       = {255, 255, 255};


struct AlgAE::FrameState
{
  typedef std::map<int, const Visible*, less<int> > KnownObjects;
  KnownObjects knownObjects;

  typedef std::list<const Visible*> VisibleList;

  VisibleList showing;
  VisibleList queue;
};



#ifndef _MSC_VER
extern "C"
void timeOut (int signal)
{
  cerr << "Timed out" << endl;
  exit (1);
}
#endif


void parseCGIArgs (const char* queryString, int& nArgs,
		   char**& args, ostream* dbgstream);


AlgAE::AlgAE (int nargs, char** args)
{
  ifstream *htmlstream = 0;
  
  int timeOutMinutes = 90;
  
  instream = 0;
  outstream = 0;
  dbgstream = 0; // &cerr;

  algorithmsMenu = 0;

  frame = new FrameState();


  
  struct  sockaddr_in server;
  socketNum = 0;
  int firstCommandArg = 0;
  

  // Get CGI parameters
  char* cgiArgs;
  if ((cgiArgs = getenv("QUERY_STRING")) != 0)
      {
	  parseCGIArgs (cgiArgs, nargs, args, dbgstream);

	  DEBUG << "extracted CGI args: ";
	  for (int j = 0; j < nargs; ++j)
	      DEBUG << " (" << args[j] << ")";
	  DEBUG << endl;
      }
  else
      {
	  // On the command line, args[0] is just the name of this program.
	  // Ignore it.
	  firstCommandArg = 1;
      }



  // Interpret command parameters
  for (int i = firstCommandArg; i < nargs; ++i)
    {
      string commandArg = args[i];
      if ((commandArg == "-infile") && ((i+1) < nargs))
	{
	  string filename = args[i+1];
	  if (filename == "-")
	    instream = &cin;
	  else
	    {
	      instream = new ifstream (filename.c_str());
	      if (!*instream)
		error (string("Could not open input file ") + filename);
	    }
	  ++i;
	}
      else if ((commandArg == "-outfile") && ((i+1) < nargs))
	{
	  string filename = args[i+1];
	  if (filename == "-")
	    outstream = &cout;
	  else
	    {
	      outstream = new ofstream (filename.c_str());
	      if (!*outstream)
		error (string("Could not open output file ") + filename);
	    }
	  ++i;
	}
      else if ((commandArg == "-htmlfile") && ((i+1) < nargs))
	{
	  string filename = args[i+1];
	  htmlstream = new ifstream (filename.c_str());
	  if (!*htmlstream)
	    error (string("Could not open html template file ") + filename);
	  ++i;
	}
      else if ((commandArg == "-debug") && ((i+1) < nargs))
	{
	  string filename = args[i+1];
	  if (filename == "-")
	    dbgstream = &cerr;
	  else
	    {
	      string debugFileName = filename;
#ifdef _MSC_VER
	      dbgstream = new ofstream (debugFileName.c_str());
#else
	      dbgstream = new ofstream (debugFileName.c_str(), ios::app);
#endif
	      if (!*dbgstream)
		error (string("Could not open debug file ") + filename);
	    }
	  ++i;
	}
      else if ((commandArg == "-port") && ((i+1) < nargs))
	{
	  istringstream portNum (args[i+1]);
	  if (!(portNum >> socketNum))
	    error (string("Port number is non-numeric ") + args[i+1]);
	  ++i;
	}
      else if ((commandArg == "-timeout") && ((i+1) < nargs))
	{
	  istringstream timeOut (args[i+1]);
	  if (!(timeOut >> timeOutMinutes))
	    error (string("Time out (minutes) is non-numeric ") + args[i+1]);
	  ++i;
	}
    }

  DEBUG << "-port " << socketNum << endl;

  int sd = 0;
    
  if ((instream == 0) || (outstream == 0))
    {// get socket info

#ifdef USE_WINSOCK
      WSADATA wsadata;
      if (WSAStartup(0x0101, &wsadata) == SOCKET_ERROR)
	{
	  cerr << "Error: could not initialize WinSock 1.1" << endl;
	  exit (-2);
	}
#endif

      if (socketNum <= 0) {
	socketNum = 0;
      }
      
	
      // Construct the socket name
      memset (&server, 0, sizeof(server));
      server.sin_family = AF_INET;
      server.sin_addr.s_addr = htonl(INADDR_ANY);
      server.sin_port = htons(socketNum);
      // Create the socket
      sd = socket (AF_INET,SOCK_STREAM,0);
  
      if (sd<0) {
	DEBUG<<"Error opening stream socket"<<endl;
	exit(-1);
      }
      DEBUG<<"Socket created" << endl;

      // And bind it to this host
      if ( bind( sd, (struct sockaddr*) &server, sizeof(server)) < 0) {
	close(sd);
	DEBUG<<"Error: " << errno
	     << " binding name to stream socket"<<endl;
	exit(-1);
      }
      DEBUG<<"Socket bound" << endl;


      socklen_t length = sizeof(server);
      if ( getsockname (sd,(struct sockaddr*) &server, &length) ) {
	cerr <<"Error: getting socket name"<<endl;
	exit(-1);
      }
      


      socketNum = ntohs(server.sin_port);
      DEBUG << "Socket number is " << socketNum << endl;
    }
  

  if (htmlstream != 0)
    {
      // Get the port number as a string
      ostringstream socketNumOut;
      socketNumOut << socketNum;
      string socketNumString = socketNumOut.str();


      //  Send the CGI output (usually an HTML page with an AlgAE
      //    client applet).

      const string socketnum = "SOCKETNUM";
      const int socketnumLen = socketnum.length();

      string line;
		  
      while (getline (*htmlstream, line))
	{
	  // Replace any occurrences of socketnum by the actual port number.
	  string::size_type pos = line.find (socketnum);
	  while (pos >= 0 && pos < line.length())
	    {
	      line.replace (pos, socketnumLen, socketNumString);
	      pos = line.find (socketnum);
	    }
      
	  cout << line << endl;
	}
      (*htmlstream).close();

      DEBUG <<"CGI output complete" << endl;
    }
  


  if ((instream == 0) || (outstream == 0))
    {
      // Close all std I/O and wait for an AlgAE client
      // to connect on the socket.

      close(0);
      close(1);
      if (dbgstream != &cerr)
	close(2);

#ifndef _MSC_VER
      //  Start up the timer.  If it expires, the server shuts down
      DEBUG<<"Starting timer " << endl;
      itimerval timeOutValue;
      itimerval dummy;
      signal (SIGALRM, timeOut);
      timeOutValue.it_value.tv_sec = 60 * timeOutMinutes;
      timeOutValue.it_value.tv_usec = 0;
      timeOutValue.it_interval.tv_sec = 0;
      timeOutValue.it_interval.tv_usec = 0;
      
      setitimer (ITIMER_REAL, &timeOutValue, &dummy);
      DEBUG<<"Timer started" << endl;
#endif

      DEBUG<<"try to listen" << endl;

      // Wait for the AlgAE client to connect to the socket.
      int listenStatus = listen(sd,1);

      DEBUG<<"Returned from listen with status " << listenStatus << endl;

      // Wait for the AlgAE client to connect to the socket.
      struct sockaddr_in from;
      socklen_t fromlen = sizeof(from);
      memset (&from, 0, fromlen);
      DEBUG<<"Accept connection from an AlgAE client" << endl;
      int socketDescr  = accept(sd, (struct sockaddr*) &from, &fromlen);
      if (socketDescr < 0)
	{
	  DEBUG <<"Error number "
		<< errno << " returned from accept call" << endl;
	  exit (-1);
	}
      
      DEBUG<<"Accepted connection,  socket descriptor="
	   << socketDescr << endl;

      instream = new iSocketStream (socketDescr);
      outstream = new oSocketStream (socketDescr);
	  
      DEBUG<<"Socket I/O streams created" << endl;
    }
      
  // open the socket
  msgs = new Messages(*instream, *outstream);
      
  DEBUG<<"Message I/O initialized" << endl;
      
  // Redirect C++ standard input and output via the message server
#ifndef __bad_rdbuf__
  cin.rdbuf (new AlgAEStreamBuffer(80, true));
  cout.rdbuf (new AlgAEStreamBuffer(80, false));
#endif
}


AlgAE::~AlgAE ()
{
  DEBUG << "AlgAE destructor invoked" << endl;
  delete msgs;
  delete frame;
  if (instream != &cin)
    delete instream;
  if (outstream != &cout)
    delete outstream;
}



  
struct AlgAEMenuItem 
{
  string menuItemTitle;
  menuFunction theOperation;
  AlgAEMenuItem* next;
  
  AlgAEMenuItem (const string& title,
		 menuFunction theOp,
		 AlgAEMenuItem* nxt)
    : menuItemTitle(title), theOperation(theOp), next(nxt)  {}

  static void invoke (const string& title, AlgAEMenuItem* menuList);
};


void AlgAEMenuItem::invoke (const string& title, AlgAEMenuItem* menuList)
{
  string str = title;
  
  if (title[0] == '"')
    str = title.substr(1, title.size()-2);
  
  //  DEBUG << "Invoke - searching for /" << title << "/" << endl;
  for (AlgAEMenuItem* current = menuList;
       current != 0; current = current->next)
    {
      // DEBUG << "Invoke - compare to /" << current->menuItemTitle << "/" << endl;
      if (current->menuItemTitle == str)
	{
	  current->theOperation();
	  break;
	}
    }
}


void AlgAE::menuItem (const string& menuItemTitle,
		      menuFunction theOperation)
{
  algorithmsMenu = new AlgAEMenuItem
    (menuItemTitle, theOperation, algorithmsMenu);
}




    
// Start the client and begin running the animation server.
void AlgAE::run(const string& animationTitle, const string& aboutString)
{
  msgs->algorithmName (animationTitle, aboutString);

  // reverse the menu list
  AlgAEMenuItem* first = 0;
  AlgAEMenuItem* next = 0;
  while (algorithmsMenu != 0)
    {
      next = algorithmsMenu->next;
      algorithmsMenu->next = first;
      first = algorithmsMenu;
      algorithmsMenu = next;
    }
  algorithmsMenu = first;

  // Send the menu list to the client
  for (next = algorithmsMenu; next != 0; next = next->next)
    msgs->menuItem (next->menuItemTitle);



  // Main loop: process client's menu selections
  string menuSelection;
  while (menuSelection != "QUIT")
    {
      msgs->beginAlgorithm();
      describeData();
      msgs->status ("Select an algorithm");
      msgs->endAlgorithm();
      menuSelection = msgs->getMenuSelection ();
      AlgAEMenuItem::invoke (menuSelection, algorithmsMenu);
    }

#ifdef USE_WINSOCK
  if (socketNum > 0)
    WSACleanup();
#endif

}


void AlgAE::error (const string& msg) const
{
  DEBUG << msg << endl;
  exit (1);
}



void AlgAE::addVisibleObject (const Visible* v)
{
  frame->knownObjects[v->id()] = v;
}

void AlgAE::removeVisibleObject (const Visible* v)
{
  frame->knownObjects.erase (v->id());
}

void AlgAE::showObject (const Visible* v)
{
  frame->showing.push_back (v);
}

void AlgAE::hideObject (const Visible* v)
{
  FrameState::VisibleList::iterator p = find(frame->showing.begin(),
					     frame->showing.end(), v);
  if (p != frame->showing.end())
    frame->showing.erase(p);
}

void AlgAE::hideObjectsSince (const Visible* v)
{
  FrameState::VisibleList::iterator p = find(frame->showing.begin(),
					     frame->showing.end(),
					     v);
  if (p != frame->showing.end())
    {
      frame->showing.erase(p, frame->showing.end());
    }
}

  

void AlgAE::enqueue(const Visible* v)
{
  if ((v != 0) && (!v->marked()))
    {
      frame->queue.push_back (v);
      v->setMark (true);
    }
}


      
void AlgAE::describeData()
{
  frame->queue = frame->showing;
  FrameState::VisibleList::iterator i;
  
  for (i = frame->queue.begin(); i != frame->queue.end(); ++i)
    (*i)->setMark(true);

  while (frame->queue.size() > 0)
    {
      const Visible* v = frame->queue.front();
      frame->queue.pop_front();

      // Get the text to display in this node
      ostringstream nodeText;
      v->writeText(nodeText);

      // Describe the node to the client
      std::string text = v->getName();
      if (text.length() > 0)
	  text.append(": ");
      text.append (nodeText.str());
      msgs->node (v->id(), v->color(), text, v->vertical());
      
      // Now, describe the edges and compound components.

      msgs->beginEdgeList();
      v->touchAllPointers();
      msgs->endEdgeList();
      
      msgs->beginComponentList();
      v->touchAllComponents();
      msgs->endComponentList();
    }

  for (FrameState::KnownObjects::iterator k = frame->knownObjects.begin();
       k != frame->knownObjects.end();
       ++k)
    {
      (*k).second->setMark(false);
    }
}


void AlgAE::unHighlightAll()
{
  for (FrameState::KnownObjects::iterator k = frame->knownObjects.begin();
       k != frame->knownObjects.end();
       ++k)
    {
      ((Visible*)((*k).second))->clearHighlighting();
    }
}



void AlgAE::FRAME (const string& frameIdentifier, const string& message)
{
  msgs->beginFrame();
  msgs->status (message);
  if (msgs->clientIsPausing())
    describeData();

  // Make sure we have permission to go on
  if (!msgs->canContinue()) {
    exit(0);
  }
  msgs->endFrame(frameIdentifier);
}

void AlgAE::FRAME (const char* frameIdentifier, const char* message)
{
    FRAME (string(frameIdentifier), string(message));
}


void AlgAE::FRAME (const char* frameIdentifier)
{
    FRAME (string(frameIdentifier), string(frameIdentifier));
}


void AlgAE::promptForInput (const string& prompt,
                            string& response)
{
  response = msgs->promptForInput (prompt);
}


void AlgAE::message (const string& msg)
{
  msgs->message (msg);
}





int getHexDigit(char c)
{
  if ((c >= '0') && (c <= '9'))
    return c - '0';
  else if ((c >= 'A') && (c <= 'F'))
    return c - 'A';
  else if ((c >= 'a') && (c <= 'f'))
    return c - 'a';
  else
    return 0;
}



/* Extract from the character array from[startPos..endPos-1] a string
   after applying the CGI conversion rules:
     All '+' characters are replaced by ' '
     All "%xx" sequences are replaced by the single character whose hex
       value is xx.
*/
string extractCGIString (const char* from, int startPos, int endPos)
{
  string out;

  for (int k = startPos; k < endPos; ++k)
    {
      if (from[k] == '+')
	out += ' ';
      else if (from[k] == '%')
	{
	  int digit1 = getHexDigit(from[k+1]);
	  int digit2 = getHexDigit(from[k+2]);
	  out += (char)(16*digit1 + digit2);
	  k += 2;
	}
      else
	out += from[k];
    }
  return out;  
}



/*  Parse a set of arguments passed via the CGI GET method, contained
    in the string queryString.  Return an array of argument strings
    (args) and a number of items in that array (nArgs) just as if the
    arguments had actually been passed via the command line.

    In particular, each CGI argument "...name=value..." is entered
    into the output array args as a pair of strings: "-name" and "value".
*/

void parseCGIArgs (const char* queryString, int& nArgs, char**& args, ostream* dbgstream)
{
  DEBUG << "parsing CGI arguments" << endl;
  int queryLen = strlen(queryString);
  
  // First, how many strings will we need in the output array?

  int nPairs = 0;
  for (int i = 0; i < queryLen; ++i)
    if (queryString[i] == '=')
      nPairs += 1;

  nArgs = 2*nPairs;
  args = new char*[nArgs];

  // Now scan the query string, extracting each argument pair
  int scanPos = 0;
  string hyphen = "-";
  string name;

  int newPosition = 0;
  for (int pairNumber = 0; pairNumber < nPairs; ++pairNumber)
    {
      // Find the position of the '=' sign
      int equalsPos = scanPos;
      while ((equalsPos < queryLen) && (queryString[equalsPos] != '='))
	++equalsPos;

      
      // Find the position of the '&' terminator
      int ampersandPos = equalsPos;
      while ((ampersandPos < queryLen) && (queryString[ampersandPos] != '&'))
	++ampersandPos;

      // Extract the strings and place in the argument array
      string name0 = extractCGIString (queryString, scanPos, equalsPos);
      name = hyphen + name0;
      string value = extractCGIString (queryString, equalsPos+1, ampersandPos);
      char* valueC = strdup(value.c_str());

      args[newPosition] = strdup (name.c_str());
      args[newPosition+1] = valueC;
      DEBUG << args[newPosition] << '=' << args[newPosition+1] << endl;
      newPosition += 2;

      scanPos = ampersandPos+1;
    }
}
