#include <algae/config.h>

#include CPPSTRING_INCL
#include CSTRING_INCL


/* Extract from the character array from[startPos..endPos-1] a string
   after applying the CGI conversion rules:
     All '+' characters are replaced by ' '
     All "%xx" sequences are replaced by the single character whose hex
       value is xx.
*/
char* extractCGIString (const char* from, int startPos, int endPos);


/*  Parse a set of arguments passed via the CGI GET method, contained
    in the string queryString.  Return an array of argument strings
    (args) and a number of items in that array (nArgs) just as if the
    arguments had actually been passed via the command line.

    In particular, each CGI argument "...name=value..." is entered
    into the output array args as a pair of strings: "-name" and "value".
*/

void parseCGIArgs (const char* queryString, int& nArgs, char**& args)
{
  int queryLen = strlen(queryString);
  
  // First, how many strings will we need in the output array?

  int nPairs = 0;
  for (int i = 0; i < queryLen; ++i)
    if (queryString[i] == '=')
      nPairs += 1;

  nArgs = 2*nPairs;
  args = new (char*)[nArgs];

  // Now scan the query string, extracting each argument pair
  int scanPos = 0;
  StdString hyphen = "-";
  StdString name;
  
  for (int pairNumber = 0; pairNumber < nPairs; ++pairNumber)
    {
      // Find the position of the '=' sign
      int equalsPos = scanPos;
      while ((equalsPos < queryLen) && (queryString[equalsPos] != '='))
	++equalsPos;

      
      // Find the position of the '&' terminator
      int ampersandPos = equalsPos;
      while ((ampersandPos < queryLen) && (queryString[ampersandPos] != '&'))
	++ampersandPos;

      // Extract the strings and place in the argument array
      char* cname = extractCGIString (queryString, scanPos, equalsPos);
      name = hyphen + cname;
      args[2*pairNumber] = strdup(name.c_str());
      delete [] cname;

      char* value = extractCGIString (queryString, equalsPos+1, ampersandPos);
      args[2*pairNumber+1] = value;

      scanPos = ampersandPos+1;
    }
}


int getHexDigit(char c)
{
  if ((c >= '0') && (c <= '9'))
    return c - '0';
  else if ((c >= 'A') && (c <= 'F'))
    return c - 'A';
  else if ((c >= 'a') && (c <= 'f'))
    return c - 'a';
  else
    return 0;
}

char* extractCGIString (const char* from, int startPos, int endPos)
{
  // Count the number of '%' chars
  int hexCnt = 0;
  for (int k = startPos; k < endPos; ++k)
    if (from[k] == '%')
      ++hexCnt;
  
  char* out = new char[endPos - startPos - 2*hexCnt + 1];

  int outPos = 0;
  for (int k = startPos; k < endPos; ++k)
    {
      if (from[k] == '+')
	out[outPos] =  ' ';
      else if (from[k] == '%')
	{
	  int digit1 = getHexDigit(from[k+1]);
	  int digit2 = getHexDigit(from[k+2]);
	  out[outPos] = (char)(16*digit1 + digit2);
	  k += 2;
	}
      else
	out[outPos] = from[k];
      ++outPos;
    }
  out[outPos] = 0;
  return out;  
}

