package edu.odu.cs.cs361.animations;


import java.awt.Color;
import java.util.ArrayList;
import java.util.List;

import edu.odu.cs.zeil.AlgAE.ActivationRecord;
import edu.odu.cs.zeil.AlgAE.Animation;
import edu.odu.cs.zeil.AlgAE.Server.MenuFunction;
import edu.odu.cs.zeil.AlgAE.Snapshot.Component;
import edu.odu.cs.zeil.AlgAE.Snapshot.Connection;
import edu.odu.cs.zeil.AlgAE.Snapshot.Rendering.CanBeRendered;
import edu.odu.cs.zeil.AlgAE.Snapshot.Rendering.Renderer;
import edu.odu.cs.zeil.AlgAE.Utilities.DiscreteInteger;

public class CS361SortingAnimation extends Animation {

	public CS361SortingAnimation() {
		super("Sorting Algorithms", true);
	}

	@Override
	public String about() {
		return "Demonstration of Sorting Algorithms,\n" +
				"prepared for CS 361, Advanced Data Strcutures\n" +
				"and Algorithms, Old Dominion University\n" +
				"Summer 2010";
	}

	
	private DiscreteInteger[] array = new DiscreteInteger[0];
	
	private class ArrayContainer implements CanBeRendered, Renderer {

		@Override
		public Renderer getRenderer() {
			return this;
		}

		@Override
		public Color getColor(Object obj) {
			return Color.white;
		}

		@Override
		public List<Component> getComponents(Object obj) {
			ArrayList<Component> c = new ArrayList<Component>();
			c.add (new Component(array));
			return c;
		}

		@Override
		public List<Connection> getConnections(Object obj) {
			return new ArrayList<Connection>();
		}

		@Override
		public int getMaxComponentsPerRow(Object obj) {
			return 1;
		}

		@Override
		public String getValue(Object obj) {
			return "";
		}
		
	}


	
	
	@Override
	public void buildMenu() {
		
		
		registerStartingAction(new MenuFunction() {
			
			@Override
			public void selected() {
				generateRandomArray(8);
				globalVar("array", new ArrayContainer());
			}
		});
		
		register ("Generate a random array", new MenuFunction() {
			@Override
			public void selected() {
				randomArrayGenerated();
			}
		});

		register ("Generate a reversed array", new MenuFunction() {
			@Override
			public void selected() {
				reverseArrayGenerated();
			}
		});
		
		register ("Insertion Sort", new MenuFunction() {
			@Override
			public void selected() {
				new Sorting().insertionSort (array, array.length);
			}
		});

		register ("Shell Sort", new MenuFunction() {
			@Override
			public void selected() {
				new Sorting().shellSort (array, array.length);
			}
		});

		register ("Merge Sort", new MenuFunction() {
			@Override
			public void selected() {
				new Sorting().mergeSort (array, array.length);
			}
		});

		register ("Quick Sort", new MenuFunction() {
			@Override
			public void selected() {
				new Sorting().quickSort (array, array.length);
			}
		});
	}
	
	public void randomArrayGenerated()
	{
		String value = promptForInput("How many elements?", "\\d+");
		int n = Integer.parseInt(value);
		generateRandomArray(n);
	}

	public void generateRandomArray(int n)
	{
		if (n != array.length) {
			array = new DiscreteInteger[n];
		}
		for (int i = 0; i < n; ++i) {
			array[i] = new DiscreteInteger((int)((double)(2*n) * Math.random()));
		}
		
	}

	
	public void reverseArrayGenerated()
	{
		String value = promptForInput("How many elements?", "\\d+");
		int n = Integer.parseInt(value);
		generateReverseArray(n);
		
		ActivationRecord arec = Animation.activate(this);//!
		arec.breakHere("generated").var("array", array).show();//!
	}

	public void generateReverseArray(int n)
	{
		if (n != array.length) {
			array = new DiscreteInteger[n];
		}
		array[n-1] = new DiscreteInteger ((int)(3.0 * Math.random()));  
		for (int i = n-2; i >= 0; --i) {
			array[i] = new DiscreteInteger(array[i+1].get() + (int)(3.0 * Math.random()));
		}
		
	}

	
	
	
	public static void main (String[] args) {
		CS361SortingAnimation demo = new CS361SortingAnimation();
		demo.runAsMain();
	}

}
