package edu.odu.cs.cs361.animations;//!

import java.awt.Color;//!

import edu.odu.cs.zeil.AlgAE.ActivationRecord;//!
import edu.odu.cs.zeil.AlgAE.Animation;//!
import edu.odu.cs.zeil.AlgAE.Utilities.DiscreteInteger;
import edu.odu.cs.zeil.AlgAE.Utilities.Index;

public class Sorting {//!

//!#ifndef SORT_H
//!#define SORT_H

//!# include <algorithm>

//
//  Based on Budd, "Data Structures in C++ using the Standard Template Library"
//      section 4.2.4
//
//!template <class EType>
public void insertionSort (DiscreteInteger[] v, int n)//!
//!void insertionSort (EType v[], unsigned int n)
// exchange the values in the vector v
// so they appear in ascending order
{
	ActivationRecord arec = Animation.activate(getClass());//!
	arec.breakHere("starting insertion sort").refParam("v", v).param("n", n).show();//!
	for (int i = 1; i < n; i++) {//!
//!    for (unsigned int i = 1; i < n; i++) {
            // move element v[i] into place
		arec.breakHere("move v[i] into place").refParam("v", v).param("n", n).var("i", new Index(i, v)).show();//!
        DiscreteInteger element = new DiscreteInteger(v[i].get());//!
//!        EType element = v[i];
		arec.breakHere("save element to move").refParam("v", v).param("n", n).var("i", new Index(i, v)).var("element", element).show();//!
        int j = i - 1;
		arec.breakHere("start looking at pos j").refParam("v", v).param("n", n).var("i", new Index(i, v)).var("element", element).var("j", new Index(j, v)).show();//!
        while (j >= 0 && element.get() < v[j].get()) {//!
//!        while (j >= 0 && element < v[j]) {
          // slide old value up
    		arec.breakHere("not there yet").refParam("v", v).param("n", n).var("i", new Index(i, v)).var("element", element).var("j", new Index(j, v)).show();//!
          v[j+1].set(v[j]);//!          v[j+1] = v[j];
          j = j - 1;
        }
		arec.breakHere("put element at j+1").refParam("v", v).param("n", n).var("i", new Index(i, v)).var("element", element).var("j", new Index(j, v)).show();//!
        // place element into position
        v[j+1].set(element); //!        v[j+1] = element;
      }
	arec.breakHere("done sorting").refParam("v", v).param("n", n).show();//!
}

///////////////////////////////

// Shellsort: sort first N items in array A
// Etype: must have copy constructor, operator=, and operator<

//!template <class Etype>
void shellSort(DiscreteInteger[] a, int n )//!
//!void shellSort(Etype a[], int n )
{
	ActivationRecord arec = Animation.activate(getClass());//!
	for (int k = 0; k < n; ++k)//!
		arec.highlight(a[k], Color.lightGray);//!
	arec.breakHere("starting Shell's sort").refParam("a", a).param("n", n).show();//!
	for(int Gap = n / 2; Gap > 0;
        Gap = Gap == 2 ? 1 : (int) (Gap / 2.2))
	{
		//inv: for all i in Gap..n-1, a[i] >= a[i-Gap]
		arec.breakHere("Gap has been chosen").refParam("a", a).param("n", n).var("Gap", Gap).show();//!
		for( int i = Gap; i < n; i++ )
		{
			for (int j = i; j >= 0; j-=Gap)//!
				arec.highlight(a[j], Color.green);//!
			arec.breakHere("'insertion sort' these elements").refParam("a", a).param("n", n).var("Gap", Gap).var("i", new Index(i,a)).show();//!
			int Tmp = a[i].get();//!
//!			Etype Tmp = a[i];
			int j = i;
			arec.breakHere("where to put Tmp?").refParam("a", a).param("n", n).var("Gap", Gap).var("i", new Index(i,a)).var("Tmp",Tmp).var("j", new Index(j,a)).show();//!
			while (j >= Gap && Tmp < a[j - Gap].get())//!
//!			while (j >= Gap && Tmp < a[j - Gap])
			{
				arec.breakHere("shift a[j-Gap] up").refParam("a", a).param("n", n).var("Gap", Gap).var("i", new Index(i,a)).var("Tmp",Tmp).var("j", new Index(j,a)).show();//!
				a[j].set(a[j - Gap]);//!
//!				a[j] = a[j - Gap];   
				j -= Gap;            
				arec.breakHere("drop down Gap positions").refParam("a", a).param("n", n).var("Gap", Gap).var("i", new Index(i,a)).var("Tmp",Tmp).var("j", new Index(j,a)).show();//!
			}
			arec.breakHere("put Tmp in a[j]").refParam("a", a).param("n", n).var("Gap", Gap).var("i", new Index(i,a)).var("Tmp",Tmp).var("j", new Index(j,a)).show();//!
			a[j].set(Tmp);//!
//!			a[j] = Tmp;
			for (int k = i; k >= 0; k-=Gap)//!
				arec.highlight(a[k], Color.lightGray);//!
		}
		arec.breakHere("The array is now " + Gap + "-sorted.").refParam("a", a).param("n", n).var("Gap", Gap).show();//!
	}
	arec.breakHere("Finished Shell's sort").refParam("a", a).param("n", n).show();//!
}



//////// Merge Sort //////////////////

// Merge routine
// 
// Given an array a of elements such that a[leftPos]..a[rightPos-1] are
// sorted, and a[rightPos]..a[rightEnd-1] are also sorted, rearranges the 
// elements so that a[leftPos..rightEnd-1] are sorted. 
//
//  E.g.
//  Before:
//   _________________________________________________________
//   | 3 | 2 | 4 | 5 | 6 | 1 | 2 | 0 | 8 | 1 | 2 | 3 | 7 | 5 |
//   ---------------------------------------------------------
//                   ^       ^           ^
//                   |       |           |
//                leftPos  rightPos    rightEnd
//  After:
//   _________________________________________________________
//   | 3 | 2 | 1 | 2 | 4 | 5 | 6 | 0 | 8 | 1 | 2 | 3 | 7 | 5 |
//   ---------------------------------------------------------
//                   ^       ^           ^
//                   |       |           |
//                leftPos  rightPos    rightEnd

//!template <class Etype>
void merge(DiscreteInteger[] a, DiscreteInteger[] tmpArray, int leftPos,//!
//!void merge(Etype a[], Etype tmpArray[], int leftPos,
		int rightPos, int rightEnd )
{
	ActivationRecord arec = Animation.activate(getClass());//!
	for (int i = leftPos; i < Math.min(rightPos, a.length); ++i)//!
		arec.highlight(a[i], Color.red);//!
	for (int i = rightPos; i < Math.min(rightEnd, a.length); ++i)//!
		arec.highlight(a[i], Color.blue);//!
	for (int i = leftPos; i < Math.min(rightEnd, a.length); ++i)//!
		arec.highlight(tmpArray[i], Color.yellow);//!
	
	arec.breakHere("starting merge").param("a", "").refParam("tmpArray", tmpArray).param("leftPos", new Index(leftPos, a)).param("rightPos", new Index(rightPos, a)).param("rightEnd", rightEnd).show();//!
	int leftEnd = rightPos;
	int tmpPos  = leftPos;
	int leftStart = leftPos;
	int numElements = rightEnd - leftPos;
	arec.breakHere("").param("a", "").refParam("tmpArray", tmpArray).param("leftPos", new Index(leftPos, a)).param("rightPos", new Index(rightPos, a)).param("rightEnd", rightEnd)//!
	  .var("leftEnd", leftEnd).var("tmpPos", new Index(tmpPos, tmpArray)).var("leftStart", leftStart).var("numElements", numElements)   .show();//!

	// Main loop.
	//   Repeatedly choose the smaller remaining element and
	//   copy into tmpArray.
	while (leftPos < leftEnd && rightPos < rightEnd) { 
		arec.breakHere("Choose smaller element").param("a", "").refParam("tmpArray", tmpArray).param("leftPos", new Index(leftPos, a)).param("rightPos", new Index(rightPos, a)).param("rightEnd", rightEnd)//!
		  .var("leftEnd", leftEnd).var("tmpPos", new Index(tmpPos, tmpArray)).var("leftStart", leftStart).var("numElements", numElements)   .show();//!
		if (a[leftPos].get() <= a[rightPos].get()) {
//!		if (a[leftPos] <= a[rightPos]) {
			arec.breakHere("copy from left").param("a", "").refParam("tmpArray", tmpArray).param("leftPos", new Index(leftPos, a)).param("rightPos", new Index(rightPos, a)).param("rightEnd", rightEnd)//!
			  .var("leftEnd", leftEnd).var("tmpPos", new Index(tmpPos, tmpArray)).var("leftStart", leftStart).var("numElements", numElements)   .show();//!
			tmpArray[tmpPos++].set(a[leftPos++]); //!
//!			tmpArray[tmpPos++] = a[leftPos++]; 
		} else  {   
			arec.breakHere("copy from right").param("a", "").refParam("tmpArray", tmpArray).param("leftPos", new Index(leftPos, a)).param("rightPos", new Index(rightPos, a)).param("rightEnd", rightEnd)//!
			  .var("leftEnd", leftEnd).var("tmpPos", new Index(tmpPos, tmpArray)).var("leftStart", leftStart).var("numElements", numElements)   .show();//!
			tmpArray[tmpPos++].set(a[rightPos++]);//!
//!			tmpArray[tmpPos++] = a[rightPos++];
		}
	}
	arec.breakHere("done working with both sections").param("a", "").refParam("tmpArray", tmpArray).param("leftPos", new Index(leftPos, a)).param("rightPos", new Index(rightPos, a)).param("rightEnd", rightEnd)//!
	  .var("leftEnd", leftEnd).var("tmpPos", new Index(tmpPos, tmpArray)).var("leftStart", leftStart).var("numElements", numElements)   .show();//!
	// Copy any elements left over from left part into tmpArray
	while (leftPos < leftEnd) {
		arec.breakHere("copy remaining left element").param("a", "").refParam("tmpArray", tmpArray).param("leftPos", new Index(leftPos, a)).param("rightPos", new Index(rightPos, a)).param("rightEnd", rightEnd)//!
		  .var("leftEnd", leftEnd).var("tmpPos", new Index(tmpPos, tmpArray)).var("leftStart", leftStart).var("numElements", numElements)   .show();//!
		tmpArray[tmpPos++].set(a[leftPos++]);//! 
//!		tmpArray[tmpPos++] = a[leftPos++]; 
	}
	arec.breakHere("done working with left").param("a","").refParam("tmpArray", tmpArray).param("leftPos", new Index(leftPos, a)).param("rightPos", new Index(rightPos, a)).param("rightEnd", rightEnd)//!
	  .var("leftEnd", leftEnd).var("tmpPos", new Index(tmpPos, tmpArray)).var("leftStart", leftStart).var("numElements", numElements)   .show();//!
	// Copy any elements left over from right part into tmpArray
	while (rightPos < rightEnd) {
		arec.breakHere("copy remaining right element").param("a","").refParam("tmpArray", tmpArray).param("leftPos", new Index(leftPos, a)).param("rightPos", new Index(rightPos, a)).param("rightEnd", rightEnd)//!
		  .var("leftEnd", leftEnd).var("tmpPos", new Index(tmpPos, tmpArray)).var("leftStart", leftStart).var("numElements", numElements)   .show();//!
		tmpArray[tmpPos++].set(a[rightPos++]);//! 
//!		tmpArray[tmpPos++] = a[rightPos++]; 
	}
	// Copy tmpArray back.
	arec.breakHere("Copy tmpArray back").param("a","").refParam("tmpArray", tmpArray).param("leftPos", new Index(leftPos, a)).param("rightPos", new Index(rightPos, a)).param("rightEnd", rightEnd)//!
	  .var("leftEnd", leftEnd).var("tmpPos", new Index(tmpPos, tmpArray)).var("leftStart", leftStart).var("numElements", numElements)   .show();//!
	tmpPos = leftStart;
	for(int i = 0; i < numElements; i++)
		a[leftStart+i].set(tmpArray[tmpPos++]);//!
//!	    a[leftStart+i] = tmpArray[tmpPos++];
	arec.breakHere("Copied").param("a","").refParam("tmpArray", tmpArray).param("leftPos", new Index(leftPos, a)).param("rightPos", new Index(rightPos, a)).param("rightEnd", rightEnd)//!
	  .var("leftEnd", leftEnd).var("tmpPos", new Index(tmpPos, tmpArray)).var("leftStart", leftStart).var("numElements", numElements)   .show();//!
}




//!template <class EType>
void m_sort (DiscreteInteger[] a, DiscreteInteger[] tmpArray,//!
//!void m_sort (EType [] a, EType [] tmpArray,
		int low, int high)//!
//!		unsigned int low, unsigned int high)
// internal merge sort routine
{
	ActivationRecord arec = Animation.activate(getClass());//!
	for (int i = low; i < Math.min(high, a.length); ++i) arec.highlight(a[i], Color.cyan);//!
	arec.breakHere("starting m_sort").param("a", "").refParam("tmpArray", tmpArray).param("low", low).param("high", high).show();//!
	if (low + 1 < high) {
		int center = (high + low) / 2;//!
//!		unsigned int center = (high + low) / 2;
		arec.breakHere("sort to left of center").param("a", "").refParam("tmpArray", tmpArray).param("low", low).param("high", high).var("center", center).show();//!
		m_sort (a, tmpArray, low, center);
		arec.breakHere("sort to right of center").param("a", "").refParam("tmpArray", tmpArray).param("low", low).param("high", high).var("center", center).show();//!
		m_sort (a, tmpArray, center, high);
		arec.breakHere("merge sorted subarrays").param("a", "").refParam("tmpArray", tmpArray).param("low", low).param("high", high).var("center", center).show();//!
		merge (a, tmpArray, low, center, high);
	}
}


//!template <class EType>
void mergeSort (DiscreteInteger [] a, int n)//!
//!void mergeSort (EType [] a, int n)
// Sort into ascending order using merge sort algorithm
{
	ActivationRecord arec = Animation.activate(getClass());//!
	arec.breakHere("starting mergeSort").param("a", "").param("n", n).show();//!
	DiscreteInteger[] tmpArray = new DiscreteInteger[n]; for (int i = 0; i < n; ++i) tmpArray[i] = new DiscreteInteger();//!
//!	Etype * tmpArray = new EType[n];
	arec.breakHere("begin recursion").param("a", "").param("n", n).refVar("tmpArray", tmpArray).show();//!
	m_sort (a, tmpArray, 0, n);
	arec.breakHere("completed recursion").param("a", "").param("n", n).refVar("tmpArray", tmpArray).show();//!
//!	delete [] tmpArray;
}



/////////// Quick sort /////////////////


//
//
//            Based upon Chapter 14 of
//            Data Structures in C++ using the STL
//            Published by Addison-Wesley, 1997
//            Written by Tim Budd, budd@cs.orst.edu
//            Oregon State University
//

//!
void swap(DiscreteInteger x, DiscreteInteger y)//!
{//!
	int tmp = x.get();//!
	x.set(y);//!
	y.set(tmp);//!
}//!
//!

//!template <class T>
int pivot (DiscreteInteger[] v,//!
		int start,//! 
		int stop,//!
		int position)//!
//!unsigned int pivot (EType v[],
//!        unsigned int start, 
//!        unsigned int stop,
//!        unsigned int position)
// partition vector into two groups
// values smaller than or equal to pivot
// values larger than pivot
// return location of pivot element
{
	ActivationRecord arec = Animation.activate(getClass());//!
	for (int k = start; k < stop; ++k) arec.highlight(v[k]);//!
	// swap pivot into starting position	
	arec.breakHere("starting pivot").param("v","").param("start", new Index(start, v)).param("stop",stop).param("position", new Index(position, v)).show();//!
	swap (v[start], v[position]);

	// partition values
	int low = start + 1;//!
	int high = stop;//!
//!	unsigned int low = start + 1;
//!	unsigned int high = stop;
	arec.breakHere("set low and high bounds").param("v","").param("start", start).param("stop",stop).param("position", position).var("low", new Index(low,v)).var("high", new Index(high,v)).show();//!
	while (low < high)
	{//!
		arec.breakHere("Do we need to swap?").param("v","").param("start", new Index(start, v)).param("stop",stop).param("position", position).var("low", new Index(low,v)).var("high", new Index(high,v)).show();//!
		if (v[low].get() < v[start].get()) {//!
//!	    if (v[low] < v[start])
			arec.breakHere("low is OK").param("v","").param("start", start).param("stop",stop).param("position", position).var("low", new Index(low,v)).var("high", new Index(high,v)).show();//!
			arec.highlight(v[low], Color.LIGHT_GRAY);//!
			low++;
		}//!
		else if (v[--high].get() < v[start].get())//!
//!		else if (v[--high] < v[start])
		{//!
			arec.highlight(v[high], Color.LIGHT_GRAY);//!
			arec.breakHere("low and high are out of place- swap").param("v","").param("start", start).param("stop",stop).param("position", position).var("low", new Index(low,v)).var("high", new Index(high,v)).show();//!
			swap (v[low], v[high]);
		} else {//!
			arec.highlight(v[high], Color.LIGHT_GRAY);//!
		}//!
		arec.breakHere("Either low or high has moved").param("v","").param("start", new Index(start, v)).param("stop",stop).param("position", position).var("low", new Index(low,v)).var("high", new Index(high,v)).show();//!
	}//!
	
	// then swap pivot back into place
	arec.breakHere("move pivot to the place where we stopped").param("v","").param("start", start).param("stop",stop).param("position", position).var("low", new Index(low,v)).var("high", new Index(high,v)).show();//!
	swap (v[start], v[--low]);
	arec.breakHere("Done with this pivot").param("v","").param("start", start).param("stop",stop).param("position", position).var("low", new Index(low,v)).var("high", high).show();//!
	return low;
}



//
//
//              Based upon Chapter 14 of
//              Data Structures in C++ using the STL
//              Published by Addison-Wesley, 1997
//              Written by Tim Budd, budd@cs.orst.edu
//              Oregon State University
//


//!template <class T>
void quickSort(DiscreteInteger[] v, int low, int high)//!
//!void quickSort(T[] v, unsigned int low, unsigned int high)
{
	ActivationRecord arec = Animation.activate(getClass());//!
	for (int k = low; k < Math.min(high, v.length); ++k) arec.highlight(v[k]);//!
	// no need to sort a vector of zero or one elements
	arec.breakHere("quicksort recursion").param("v","").param("low", low).param("high",high).show();//!
	if (low >= high -1)
		return;

	// select the pivot value
	int pivotIndex = (low + high) / 2;//!
//!	unsigned int pivotIndex = (low + high) / 2;

	arec.breakHere("pivot on the middle value").param("v","").param("low", low).param("high",high).var("pivotIndex",new Index(pivotIndex,v)).show();//!
	// partition the vector
	pivotIndex = pivot (v, low, high, pivotIndex);

	arec.breakHere("pivot completed").param("v","").param("low", low).param("high",high).var("pivotIndex",new Index(pivotIndex,v)).show();//!
	// sort the two sub vectors
	if (low < pivotIndex) 
	{//!
		arec.breakHere("sort the low part").param("v","").param("low", new Index(low, v)).param("high",high).var("pivotIndex",new Index(pivotIndex,v)).show();//!
		quickSort(v, low, pivotIndex);
	}//!
	if (pivotIndex < high) 
	{//!
		arec.breakHere("sort the high part").param("v","").param("low", low).param("high", new Index(high, v)).var("pivotIndex",new Index(pivotIndex,v)).show();//!
		quickSort(v, pivotIndex + 1, high);
	}//!
}

//!template <class T> 
void quickSort(DiscreteInteger[] v, int numberOfElements)//!
//!void quickSort(T[] v, unsigned int numberOfElements)
{
	ActivationRecord arec = Animation.activate(getClass());//!
	arec.breakHere("kick off the recursion").refParam("v", v).param("numberOfElements", numberOfElements).show();//!
	if (numberOfElements > 1) 
		quickSort(v, 0, numberOfElements); // note: mistake in Budd, p 353
	arec.breakHere("finished quicksort").param("v","").param("numberOfElements", numberOfElements).show();//!
}




//!#endif  
        
        
}//!
