package edu.odu.cs.zeil.AlgAE;

import java.util.concurrent.ArrayBlockingQueue;

import edu.odu.cs.zeil.AlgAE.Client.Client;
import edu.odu.cs.zeil.AlgAE.Client.Message;
import edu.odu.cs.zeil.AlgAE.Server.Server;
import edu.odu.cs.zeil.AlgAE.Snapshot.ActivationStack;

/**
 *  Each animation is characterized by a small set of
 *  "singleton" objects.
 *  
 *  Note that if the animation is being run as an applet,
 *  then there may be several animations in progress simultaneously.
 *  So these are singletons in the sense that there is exactly one of 
 *  these per animation.  The information they contain must be kept distinct
 *  from one animation to another.
 *
 *  @author Steven J Zeil
 **/
public interface AnimationContext
{

	/**
	 * Each animation has a unique activation stack.
	 * @return the activation stack for this animation
	 */
	public ActivationStack getActivationStack();
	
	/**
	 * Each animation has a unique client.
	 * @return the client for this animation
	 */
	public Client getClient ();

	/**
	 * Each animation has a unique server.
	 * @return the server for this animation
	 */
	public Server getServer ();

	/**
	 * Each animation has a queue of messages used to send
	 * info from the server to the client.
	 * 
	 * @return the message queue for this animation
	 */
	public ArrayBlockingQueue<Message> getMessageQueue ();

	
}



