package edu.odu.cs.zeil.AlgAE.Client.IOViewer;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import edu.odu.cs.zeil.AlgAE.Animation;

public class IOPane extends JPanel
{
  private JTextArea stdOutputArea;
  private JTextField stdInputArea;
  
  private PipedInputStream piOut;
  private PipedOutputStream poOut;
   
  private PipedInputStream piIn;
  private PipedOutputStream poIn;
  
  public PrintStream out;

  public IOPane(Animation anim)
  {
    super();
	setLayout (new BorderLayout());
	stdOutputArea = new JTextArea();
	stdOutputArea.setEditable(false);
	stdOutputArea.setRows(12);
	add(new JScrollPane(stdOutputArea), BorderLayout.CENTER);
	
	stdInputArea = new JTextField();
	stdInputArea.setEditable(true);
	add (stdInputArea, BorderLayout.SOUTH);
	stdInputArea.addActionListener(new ActionListener() {
		
		@Override
		public void actionPerformed(ActionEvent e) {
			String inputText = stdInputArea.getText();
			System.out.println(inputText);
			inputText = inputText + "\n";
			stdInputArea.setText("");
			byte[] inputBytes = inputText.getBytes();
			try {
				poIn.write (inputBytes);
			} catch (IOException ex) {
				stdOutputArea.append("\nCould not process input string: " + ex);
			}
		}
	});
	
	// Substitute for System.out
	try {
		piOut = new PipedInputStream();
		poOut = new PipedOutputStream(piOut);
		out = new PrintStream(poOut, true);
	} catch (IOException ex) {
		stdOutputArea.append("Could not create output stream for this console.\n");
	}

	
	// Substitute for System.in
	try {
		piIn = new PipedInputStream();
		poIn = new PipedOutputStream(piIn);
		anim.in = new BufferedInputStream(piIn);
	} catch (IOException ex) {
		stdOutputArea.append("Could not create input stream for this console.\n");
	}

	Thread sysoutThread = new ReaderThread(piOut);
	sysoutThread.start();

  }


 
  
  public void setFontSize (int size)
  {
    Font textFont = new Font("Courier", Font.PLAIN, size);
    stdOutputArea.setFont (textFont);
    stdInputArea.setFont (textFont);
    repaint();
  }


  public Dimension getPreferredSize() {
    return new Dimension (250,50);
  }

  
  class ReaderThread extends Thread {
		PipedInputStream pi;

		ReaderThread(PipedInputStream pi) {
			this.pi = pi;
		}

		public void run() {
			final byte[] buf = new byte[1024];
			try {
				while (true) {
					final int len = pi.read(buf);
					if (len == -1) {
						break;
					}
					stdOutputArea.append(new String(buf, 0, len));

					// Make sure the last line is always visible
					stdOutputArea.setCaretPosition(stdOutputArea.getDocument().getLength());

					// Keep the text area down to a certain character size
					int maxSize = 50000;
					int consoleSize = stdOutputArea.getDocument().getLength();
					if (consoleSize >= maxSize) {
						stdOutputArea.replaceRange("", 0, maxSize/2);
					}
				}
			} catch (IOException e) {
			}
		}
  }


  
  
  
}
